signature ABSYN =
sig
    type absyn
		 
	val makeVar : string -> absyn
	val makeInt : int -> absyn
	val makePair : (absyn * absyn) -> absyn
	val makeLam : (string * absyn) -> absyn
	val makeApp : (absyn * absyn) -> absyn
	val makeLet : (string * absyn * absyn) -> absyn

	val print_ast : absyn -> string
end

structure Absyn :> ABSYN =
struct
datatype absyn =
		 Var of string
	   | Int of int
	   | Pair of (absyn * absyn)
	   | Lam of (string * absyn)
	   | App of (absyn * absyn)
	   | Let of (string * absyn * absyn)

fun makeVar (s : string) = Var s
fun makeInt (i : int) = Int i
fun makePair (e1, e2) = Pair (e1, e2)
fun makeLam (v, e1) = Lam (v, e1)
fun makeApp (e1, e2) = App (e1, e2)
fun makeLet (v, e1, e2) = Let (v, e1, e2)

fun print_ast (Var v) = v
  | print_ast (Int i) = Int.toString i
  | print_ast (Pair (e1, e2)) = "(" ^ (print_ast e1) ^ ", " ^ (print_ast e2) ^ ")"
  | print_ast (Lam (x, e)) = "(lam " ^ x ^ " . " ^ (print_ast e) ^ ")"
  | print_ast (App (e1, e2)) = "(" ^ (print_ast e1) ^ " " ^ (print_ast e2) ^ ")"
  | print_ast (Let (x, e1, e2)) = "let " ^ x ^ " = " ^ (print_ast e1) ^ " in " ^ (print_ast e2)
end
